/*
 * Copyright (C) 2008 by egnite GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*!
 * $Id$
 */

/*!
 * \example eirhwt/eirhwt.c
 *
 */

#include "eirhwt.h"

static char *version = "0.1.1";

unsigned char mac[6] = { 0x00, 0x06, 0x98, 0x14, 0x99, 0x99 };

/*
 * Main application routine. 
 *
 */
int main(void)
{
    int fail = 0;
    unsigned long cpuclk;
    unsigned long ram_size;

    DbgInit();

    /*
     * The timer functions automatically determine the
     * CPU speed during system initialization. Query that
     * value and print it on the console.
     */
    printf("\n\nEIR Hardware Tester %s\n", version);

    cpuclk = NutGetCpuClock();
    printf("CPU Clock: %u.%04u MHz\n", (int) (cpuclk / 1000000UL), (int) ((cpuclk - (cpuclk / 1000000UL) * 1000000UL) / 100));

    /*
     * Test MMC
     */
    printf("MMC Init...");
    if(MmCardInit()) {
        fail++;
        puts("Error");
    }
    else {
        puts("OK");
        QueryCard();
    }

    /*
     * Test Ethernet.
     */
    if (DmInit(mac)) {
        fail++;
        puts("No Ethernet");
    }

    if (Vs10x3Check()) {
        puts("No Codec");
        fail++;
    }

    /*
     * Test Dataflash.
     */
    if (TestDataFlash()) {
        puts("No DataFlash");
        fail++;
    }

    /*
     * Test RAM
     */
    ram_size = SdramTest();
    if (ram_size) {
        printf("%lu bytes SDRAM\n", ram_size);
    }
    else {
        puts("No RAM");
        fail++;
    }

    /*
     * Endless application loop.
     */
    for (;;) {
        if (fail) {
            puts("Bad EIR");
        }
        else {
            puts("EIR OK");
        }
        Delay(255); Delay(255); Delay(255); Delay(255);
        Delay(255); Delay(255); Delay(255); Delay(255);
        Delay(255); Delay(255); Delay(255); Delay(255);
        Delay(255); Delay(255); Delay(255); Delay(255);
    }
    return 0;
}
