/*
 * Copyright (C) 2008 by egnite GmbH. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
 * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.ethernut.de/
 */

/*!
 * $Id$
 */
#include <stdlib.h>
#include <string.h>

#include "eirhwt.h"

#define SDRAM_BASE  0x20000000
#define SDRAM_AREA  0x10000000
#define MEGA_BYTE   (1024 * 1024)

#define MERR_BAD_ADDR       1
#define MERR_BAD_DATA       2
#define MERR_STICKY_WRITE   3
#define MERR_STICKY_READ    4
#define MERR_STICKY_ALE     5
#define MERR_STICKY_DATABUS 6
#define MERR_STICKY_ADDRBUS 7
#define MERR_SHORT_DATABUS  8
#define MERR_SHORT_ADDRBUS  9

typedef unsigned __PTRDIFF_TYPE__ uptr_t;

static int merr;
static unsigned long merr_bits;
static void *merr_addr;

unsigned long SdramTest(void)
{
    volatile unsigned long *mem;
    unsigned long addr;
    unsigned long wb = 1;
    unsigned long siz;

    printf("Clear area  ");
    for (addr = SDRAM_BASE; addr < SDRAM_BASE + SDRAM_AREA; addr += 4 * MEGA_BYTE) {
        putchar('.');
        memset((void *)addr, 0, 4 * MEGA_BYTE);
    }
    puts("OK");

    printf("Pattern test");
    for (siz = 0; siz < SDRAM_AREA; siz += 1024) {
        mem = (unsigned long *)(SDRAM_BASE + siz);
        if (*mem) {
            break;
        }
        *mem = 0x55555555;
        if (*mem != 0x55555555) {
            break;
        }
        *mem = 0xaaaaaaaa;
        if (*mem != 0xaaaaaaaa) {
            break;
        }
        if ((siz & (MEGA_BYTE - 1)) == 0) {
            putchar('.');
        }
    }
    puts("OK");

    printf("Fill SDRAM  ");
    for (addr = SDRAM_BASE; addr < SDRAM_BASE + siz; addr += MEGA_BYTE) {
        putchar('.');
        memset((void *)addr, 0xff, MEGA_BYTE);
    }
    puts("OK");

    printf("Test SDRAM  ");
    for (mem = (unsigned long *)SDRAM_BASE; siz && mem < (unsigned long *)(SDRAM_BASE + siz); mem++) {
        if (((unsigned long)mem & (unsigned long)(MEGA_BYTE - 1)) == 0) {
            putchar('.');
        }
        if (*mem + 1) {
            break;
        }
        wb = 1;
        *mem = wb;
        do {
            if(*mem != wb) {
                siz = 0;
                break;
            }
            wb <<= 1;
            *mem = wb;
        } while(wb);
    }
    if (siz) {
        puts("OK");
    }
    else {
        puts("Error");
        if(wb) {
            merr_bits = *mem & ~wb;
        }
        else {
            merr_bits = *mem;
        }
        merr_addr = (void *)mem;
        merr = MERR_BAD_DATA;
        siz = 0;
    }
    return siz;
}

